import { ref, watch, onBeforeMount, Ref, computed, nextTick } from "vue";
import * as API from "@/api";
import { Howl, Howler } from "howler";
import { useRoute } from "vue-router";

//答题模式切换
export function useTopicMode() {
  const answerTypeList = ref([
    { name: "顺序练习" },
    { name: "随机练习" },
    { name: "背题模式" },
  ]);
  const currentType = ref(0);
  const typeParams = ref({
    order: true, //顺序练习
    answerShow: false, //背题模式
  });
  watch(currentType, (currentVal) => {
    switch (currentVal) {
      case 0:
        typeParams.value.order = true;
        typeParams.value.answerShow = false;
        break;
      case 1:
        typeParams.value.order = false;
        typeParams.value.answerShow = false;
        break;
      case 2:
        typeParams.value.order = true;
        typeParams.value.answerShow = true;
        break;
    }
  });
  return {
    answerTypeList,
    currentType,
    typeParams,
  };
}

//语音设置
export function useAudioSet(currentAnswerIndex: Ref<number>) {
  const aotuPlayFlag = ref(false);

  let sound: Howl;
  /**
   * 播放音频
   * @param audioUrl
   */
  const audioPlay = (audioUrl: string | string[]) => {
    audioPause();
    sound = new Howl({
      src: audioUrl,
    });
    sound.once("load", function () {
      sound.play();
    });
    if (typeof audioUrl === "object") {
      sound.once("end", () => {
        sound = new Howl({
          src: audioUrl[1],
        });
        sound.once("load", function () {
          sound.play();
        });
      });
    }
  };

  /**
   * 自动读题
   */
  const issueAudioPlay = () => {};

  /**
   * 停止播放
   */
  const audioPause = () => {
    sound && sound.pause();
  };

  //音频模块end
  const aotuPlaySet = () => {
    aotuPlayFlag.value = !aotuPlayFlag.value;
    if (aotuPlayFlag.value) issueAudioPlay();
    if (!aotuPlayFlag.value) audioPause();
  };

  onBeforeMount(() => {
    watch(currentAnswerIndex, () => {
      if (aotuPlayFlag.value) issueAudioPlay(); //自动读题
    });
  });

  return {
    aotuPlayFlag,
    issueAudioPlay,
    audioPause,
    aotuPlaySet,
    audioPlay,
  };
}

const useSubjectList = () => {
  const subjectList = ref<any[]>([]); //题目列表
  const subjectTotal = ref(0); //题目总数
  const pageNum = ref(1); //当前请求页码
  const query = useRoute().query; //路由query参数
  onBeforeMount(async () => {
    const res = await API.getTopicList({
      ...query,
      pageNum: pageNum.value,
      pageSize: 10,
    });
    subjectList.value = res.list;
    subjectTotal.value = res.total;
  });
  //加载下一页数据
  const loadNewSubject = async () => {
    if (subjectList.value.length == subjectTotal.value) return;
    pageNum.value++;
    const res = await API.getTopicList({
      ...query,
      pageNum: pageNum.value,
      pageSize: 10,
    });
    subjectList.value = subjectList.value.concat(res.list);
  };
  const currentSubjectIndex = ref(0); //当前题目下标
  //当前题目内容
  const currentSubject = computed(() => {
    return subjectList.value[currentSubjectIndex.value];
  });
  return {
    subjectList,
    subjectTotal,
    loadNewSubject,
    currentSubject,
    currentSubjectIndex,
  };
};

export const useSubjectShowLogic = () => {
  const trueNum = ref(0); //正确数量
  const falseNum = ref(0); //错误数量
  const isJumpNext = ref(false); //答对跳转下一题
  const {
    subjectList,
    subjectTotal,
    loadNewSubject,
    currentSubject,
    currentSubjectIndex,
  } = useSubjectList(); //获取题目列表

  const nextBtnState = ref(true); //下一题数据请求锁
  /**
   * 展示下一题
   */
  const nextSubject = async () => {
    if (currentSubjectIndex.value < subjectList.value.length - 1) {
      currentSubjectIndex.value++;
    } else {
      if (nextBtnState.value) {
        //禁用下一题按钮
        nextBtnState.value = false;
        //题目数量不足加载数据
        await loadNewSubject();
        //启用按钮
        nextBtnState.value = true;
        nextSubject()
      }
    }
  };
  /**
   * 展示上一题
   */
  const lastSubject = () => {
    if (currentSubjectIndex.value > 0) {
      currentSubjectIndex.value--;
    }
  };
  /**
   * 题目类型判断
   * @param type
   * @returns
   */
  const topicType = (type: Number) => {
    switch (type) {
      case 0:
        return "判断题";
      case 1:
        return "单选题";
      case 2:
        return "多选题";
    }
  };

  //选择答案后
  const userAnswerChange = () => {
    currentSubject.value.optsBack = currentSubject.value.opts.map(
      (val: String) => {
        let status;
        if (currentSubject.value.answer.includes(val)) {
          status = 1;
        } else {
          status = 0;
        }
        if (currentSubject.value.userAnswer.includes(val)) {
          status += 2;
        }
        return { opt: val, status };
      }
    );
    if (
      JSON.stringify(currentSubject.value.answer) ==
      JSON.stringify(currentSubject.value.userAnswer)
    ) {
      console.log("答案正确");
      currentSubject.value.isTrue = true;
      trueNum.value++;
      if (isJumpNext.value) {
        nextTick(() => {
          nextSubject();
        });
      }
    } else {
      console.log("错误");
      currentSubject.value.isTrue = false;
      falseNum.value++;
    }
  };

  return {
    currentSubject,
    currentSubjectIndex,
    subjectTotal,
    nextSubject,
    lastSubject,
    trueNum,
    falseNum,
    isJumpNext,
    userAnswerChange,
    topicType,
  };
};
